@@ -7,16 +7,16 @@ import os
 import shutil
 import warnings
 from collections import defaultdict
+from datetime import datetime
 from pathlib import Path, PosixPath
-from importlib import reload
 from textwrap import dedent
 from uuid import uuid4
-
 import pyrsistent as pyrs
 import pandas as pd
 
 # A bit annoying that we have to rename this when we import it.
 from . import protocols as protos
+from .aip.task import Config as AipConfig
 from .cache_api import Cache
 from .datatypes import (
     CaseKey,
@@ -31,7 +31,7 @@ from .exception import (
     UnsetEntityError,
     AttributeValidationError,
 )
-from .executor import Executor
+from .executor import AipExecutor, ProcessExecutor
 from .persistence import LocalStore, GcsCloudStore, PersistentCache
 from .provider import (
     ValueProvider,
@@ -42,23 +42,27 @@ from .deriver import EntityDeriver, entity_is_internal
 from .descriptors.parsing import entity_dnode_from_descriptor
 from . import decorators, decoration
 from .filecopier import FileCopier
-from .gcs import copy_to_gcs
+from .gcs import upload_to_gcs
 from .utils.misc import (
     group_pairs,
     check_exactly_one_present,
     check_at_most_one_present,
     oneline,
 )
+from .utils.reload import recursive_reload
+from .utils.urls import path_from_url
 
 DEFAULT_PROTOCOL = protos.CombinedProtocol(
     protos.JsonProtocol(),
     # Every GeoPandas DataFrame is also a Pandas DataFrame, so we need to do the
     # GeoPandas check first.
     protos.GeoPandasProtocol(),
+    # Similarly, every Dask file extension also matches the Parquet extension, so we
+    # need to check Dask first.
+    protos.DaskProtocol(),
     protos.ParquetDataFrameProtocol(),
     protos.ImageProtocol(),
     protos.NumPyProtocol(),
-    protos.DaskProtocol(),
     protos.PicklableProtocol(),
 )
 
@@ -249,7 +253,7 @@ class FlowState(pyrs.PClass):
         tokens = []
         for name, value in zip(names, values):
             protocol = self.get_entity_def(name).protocol
-            protocol.validate(value)
+            protocol.validate_for_entity(name, value)
             tokens.append(protocol.tokenize(value))
         provider = provider.add_case(case_key, values, tokens)
 
@@ -303,13 +307,17 @@ class FlowState(pyrs.PClass):
         assert name not in self.providers_by_name
 
         state = self
-        state = state.set(entity_defs_by_name=state.entity_defs_by_name.remove(name),)
+        state = state.set(
+            entity_defs_by_name=state.entity_defs_by_name.remove(name),
+        )
         return state
 
     def _erase_provider(self, name):
         state = self
         if name in state.providers_by_name:
-            state = state.set(providers_by_name=state.providers_by_name.remove(name),)
+            state = state.set(
+                providers_by_name=state.providers_by_name.remove(name),
+            )
         if name in state.default_entity_names:
             state = state.set(
                 default_entity_names=(state.default_entity_names.remove(name))
@@ -485,10 +493,6 @@ class FlowBuilder:
             )
             doc = docstring
 
-        # TODO We can remove this validation, since it also happens in add_case below.
-        for value in values:
-            protocol.validate(value)
-
         state = self._state
 
         state = state.define_entity(
@@ -537,9 +541,6 @@ class FlowBuilder:
         protocol = state.get_entity_def(name).protocol
 
         for value in values:
-            # TODO We can remove this validation, since it also happens in add_case
-            # below.
-            protocol.validate(value)
             case_key = CaseKey([(name, protocol.tokenize(value))])
             state = state.add_case(case_key, [name], [value])
 
@@ -629,7 +630,7 @@ class FlowBuilder:
             protocol = state.get_entity_def(name).protocol
             # TODO Both the validation and tokenization are also happening in
             # state.add_case below; maybe we can remove some duplicate work.
-            protocol.validate(value)
+            protocol.validate_for_entity(name, value)
             token = protocol.tokenize(value)
 
             values.append(value)
@@ -985,6 +986,9 @@ class FlowBuilder:
                 doc=doc,
                 optional_should_persist=acc.should_persist,
                 optional_should_memoize=acc.should_memoize,
+                # If this entity is derived in a non-deterministic way, this flag will
+                # remind us to check and make sure it actually gets cached somehow.
+                needs_caching=provider.attrs.changes_per_run,
             )
             state = state.define_entity(entity_def)
         state = state.install_provider(provider)
@@ -1207,10 +1211,9 @@ class Flow:
         if mode is object or mode == "object":
             values = [result.value for result in result_group]
         else:
-            # all other modes expect the entity to be persisted
-            result_file_paths = [result.file_path for result in result_group]
-
-            if None in result_file_paths:
+            # All other modes expect the entity to be persisted.
+            result_artifacts = [result.local_artifact for result in result_group]
+            if None in result_artifacts:
                 raise ValueError(
                     oneline(
                         f"""
@@ -1218,13 +1221,16 @@ class Flow:
                     expected by mode {mode!r}"""
                     )
                 )
+            result_paths = [
+                path_from_url(artifact.url) for artifact in result_artifacts
+            ]
 
             if mode is Path or mode == "path":
-                values = result_file_paths
+                values = result_paths
             elif mode == "FileCopier":
-                values = [FileCopier(fp) for fp in result_file_paths]
+                values = [FileCopier(path) for path in result_paths]
             elif mode == "filename":
-                values = [str(fp) for fp in result_file_paths]
+                values = [str(path) for path in result_paths]
             else:
                 raise ValueError(f"Unrecognized mode {mode!r}")
 
@@ -1244,7 +1250,7 @@ class Flow:
                 # missing values.
                 assert len(orig_result_group) > 0
                 missing_result = orig_result_group[0]
-                missing_names = missing_result.query.case_key.missing_names
+                missing_names = missing_result.task_key.case_key.missing_names
                 message = f"""
                 Entity {name} could not be computed because the following entities are
                 declared but have no values set:
@@ -1271,12 +1277,12 @@ class Flow:
                         ancestor_name
                     ] = ancestor_result_group.key_space
                     for ancestor_result in ancestor_result_group:
-                        ancestor_case_key = ancestor_result.query.case_key
+                        ancestor_case_key = ancestor_result.task_key.case_key
                         ancestor_values_by_case_key_by_name[ancestor_name][
                             ancestor_case_key
                         ] = ancestor_result.value
 
-                orig_case_keys = [result.query.case_key for result in result_group]
+                orig_case_keys = [result.task_key.case_key for result in result_group]
                 index = pd.MultiIndex.from_tuples(
                     tuples=[
                         tuple(
@@ -1298,7 +1304,11 @@ class Flow:
                 )
             else:
                 index = None
-            return pd.Series(name=name, data=values, index=index,)
+            return pd.Series(
+                name=name,
+                data=values,
+                index=index,
+            )
         else:
             raise ValueError(f"Unrecognized collection type {collection!r}")
 
@@ -1342,9 +1352,9 @@ class Flow:
 
         (result,) = result_group
 
-        if result.file_path is None:
+        if result.local_artifact is None:
             raise ValueError(f"Entity {name!r} is not locally persisted")
-        src_file_path = result.file_path
+        src_file_path = path_from_url(result.local_artifact.url)
 
         if dir_path is None and file_path is None:
             return src_file_path
@@ -1366,7 +1376,7 @@ class Flow:
 
         if dst_file_path_str.startswith("gs:/"):
             # The path object combines // into /, so we revert it here
-            copy_to_gcs(str(src_file_path), dst_file_path_str.replace("gs:/", "gs://"))
+            upload_to_gcs(src_file_path, dst_file_path_str.replace("gs:/", "gs://"))
         else:
             shutil.copyfile(str(src_file_path), dst_file_path_str)
 
@@ -1429,7 +1439,13 @@ class Flow:
 
         return self.get("core__flow_name")
 
-    def render_dag(self, include_core=False, vertical=False, curvy_lines=False):
+    def render_dag(
+        self,
+        include_core=False,
+        vertical=False,
+        curvy_lines=False,
+        _include_detail=False,
+    ):
         """
         Returns a ``FlowImage`` with a visualization of this flow's DAG. This
         object behaves similarly to a Pillow ``Image`` object.
@@ -1439,18 +1455,22 @@ class Flow:
 
         from . import dagviz
 
-        graph = self._deriver.export_dag(include_core)
+        graph = self._deriver.export_dag(
+            include_core=include_core,
+            _include_detail=_include_detail,
+        )
         dot = dagviz.dot_from_graph(
-            graph=graph, vertical=vertical, curvy_lines=curvy_lines, name=self.name,
+            graph=graph,
+            vertical=vertical,
+            curvy_lines=curvy_lines,
+            name=self.name,
         )
         return dagviz.FlowImage(dot)
 
-    # TODO Should we offer an in-place version of this?  It's contrary to the
-    # idea of an immutable API, but it might be more natural for the user, and
-    # reloading is already updating global state....
-    def reloading(self):
+    def reload(self):
         """
-        Attempts to reload all modules used directly by this flow.
+        Attempts to reload all modules used directly by this flow, updates this
+        flow instance in place, and then returns the flow instance.
 
         For safety, this only works if this flow meets the following
         requirements:
@@ -1460,6 +1480,10 @@ class Flow:
         * is assigned to a top-level variable in a module that one of its
           functions is defined in
 
+        You will need to use versioning to ensure that any code changes are
+        detected properly. Otherwise, the flow may keep using cached values
+        from previous versions of the code.
+
         The most straightforward way to meet these requirements is to define
         your flow in a module as:
 
@@ -1480,10 +1504,29 @@ class Flow:
 
             from mymodule import flow
             ...
-            flow.reloading().get('my_entity')
+            flow.reload()
+            flow.get('my_entity')
 
-        This will reload the modules and use the most recent version of the
-        flow before doing the ``get()``.
+        This will update the flow instance to use the reloaded modules before
+        doing the ``get()``.
+
+        This resets the flow's in-memory cache. Entities that have not been
+        persisted to disk, or that have been marked with ``@changes_per_run``,
+        will be recomputed.
+        """
+        flow = self.reloading()
+        self._set_state(flow._state)
+        return self
+
+    def reloading(self):
+        """
+        Returns a new copy of this flow in in which all the modules used
+        directly by the flow are reloaded.
+
+        This method is similar to the ``reload()`` method, but the existing flow
+        instance remains unchanged.
+
+        Please see the comments on ``reload()`` method for safety requirements.
         """
 
         # TODO If we wanted, I think we could support reloading on modified
@@ -1504,28 +1547,39 @@ class Flow:
 
         self_name = self.name
 
-        module_names = set()
+        # Find the module that contains the flow.
+        candidate_flow_modules = set()
         for provider in state.providers_by_name.values():
             source_func = provider.get_source_func()
             if source_func is None:
                 continue
-            module_names.add(source_func.__module__)
+            module = module_registry[source_func.__module__]
+            if len(self._get_flows_from_module(module)) > 0:
+                candidate_flow_modules.add(module)
+        if len(candidate_flow_modules) == 0:
+            raise Exception(
+                oneline(f"Couldn't find the module that has flow {self_name!r}.")
+            )
+        if len(candidate_flow_modules) > 1:
+            raise Exception(
+                oneline(
+                    f"""
+                        Too many modules that contain flow {self_name!r},
+                        found: {len(candidate_flow_modules)}, wanted 1"""
+                )
+            )
+        (flow_module,) = candidate_flow_modules
 
+        flow_module = recursive_reload(flow_module)
+
+        flows = self._get_flows_from_module(flow_module)
         blessed_candidate_flows = []
         unblessed_candidate_flows = []
-        for module_name in module_names:
-            module = reload(module_registry[module_name])
-            for key in dir(module):
-                element = getattr(module, key)
-                if not isinstance(element, Flow):
-                    continue
-                flow = element
-                if flow.name != self_name:
-                    continue
-                if not flow._state.is_blessed:
-                    unblessed_candidate_flows.append(flow)
-                else:
-                    blessed_candidate_flows.append(flow)
+        for flow in flows:
+            if not flow._state.is_blessed:
+                unblessed_candidate_flows.append(flow)
+            else:
+                blessed_candidate_flows.append(flow)
 
         if len(blessed_candidate_flows) == 0:
             if len(unblessed_candidate_flows) > 0:
@@ -1541,7 +1595,7 @@ class Flow:
                     oneline(
                         f"""
                     Couldn't find any flow named {self_name!r}
-                    in modules {module_names!r}"""
+                    in module {flow_module.__name__!r}"""
                     )
                 )
         if len(blessed_candidate_flows) > 1:
@@ -1549,7 +1603,7 @@ class Flow:
                 oneline(
                     f"""
                 Too many flows named {self_name!r}
-                in modules {module_names!r};
+                in module {flow_module.__name__!r};
                 found {len(blessed_candidate_flows)}, wanted 1"""
                 )
             )
@@ -1570,9 +1624,7 @@ class Flow:
                 "use one of the classmethod constructors"
             )
 
-        self._uuid = str(uuid4())
-        self._state = state
-        self._deriver = EntityDeriver(state, self._uuid)
+        self._set_state(state)
 
         # We replace the `get` and `setting` methods with wrapper classes
         # that have an attribute for each entity.  This allows a convenient,
@@ -1594,8 +1646,6 @@ class Flow:
 
         self.get = GetMethod()
 
-        self.cache = Cache(self._deriver)
-
         class SettingMethod(ShortcutProxy):
             __doc__ = orig_setting_method.__doc__
 
@@ -1611,11 +1661,29 @@ class Flow:
 
         self.setting = SettingMethod()
 
+    def _set_state(self, state):
+        self._state = state
+        self._uuid = str(uuid4())
+        self._deriver = EntityDeriver(self._state, self._uuid)
+        self.cache = Cache(self._deriver)
+
     def _updating(self, builder_update_func):
         builder = FlowBuilder._from_state(self._state)
         builder_update_func(builder)
         return Flow._from_state(builder._state)
 
+    def _get_flows_from_module(self, module):
+        flows = []
+        for key in dir(module):
+            element = getattr(module, key)
+            if not isinstance(element, Flow):
+                continue
+            flow = element
+            if flow.name != self.name:
+                continue
+            flows.append(flow)
+        return flows
+
 
 class ShortcutProxy:
     """
@@ -1696,15 +1764,18 @@ def create_default_flow_state():
     def core__versioning_policy(core__versioning_mode):
         if core__versioning_mode == "manual":
             return VersioningPolicy(
-                treat_bytecode_as_functional=False, check_for_bytecode_errors=False,
+                treat_bytecode_as_functional=False,
+                check_for_bytecode_errors=False,
             )
         elif core__versioning_mode == "assist":
             return VersioningPolicy(
-                treat_bytecode_as_functional=False, check_for_bytecode_errors=True,
+                treat_bytecode_as_functional=False,
+                check_for_bytecode_errors=True,
             )
         elif core__versioning_mode == "auto":
             return VersioningPolicy(
-                treat_bytecode_as_functional=True, check_for_bytecode_errors=False,
+                treat_bytecode_as_functional=True,
+                check_for_bytecode_errors=False,
             )
         else:
             raise ValueError(
@@ -1750,21 +1821,24 @@ def create_default_flow_state():
 
     @builder
     @decorators.immediate
-    def core__persistent_cache__local_store(core__persistent_cache__flow_dir,):
+    def core__persistent_cache__local_store(
+        core__persistent_cache__flow_dir,
+    ):
         local_flow_dir = core__persistent_cache__flow_dir
         return LocalStore(local_flow_dir)
 
     @builder
     @decorators.immediate
     def core__persistent_cache__cloud_store(
-        core__persistent_cache__gcs__url, core__persistent_cache__gcs__enabled,
+        core__persistent_cache__gcs__url,
+        core__persistent_cache__gcs__enabled,
     ):
         gcs_url = core__persistent_cache__gcs__url
         gcs_enabled = core__persistent_cache__gcs__enabled
         if gcs_enabled:
             if gcs_url is None:
                 raise AssertionError(
-                    "core__persistent_cache__gcs__url is None, " "but needs a value"
+                    "core__persistent_cache__gcs__url is None, but needs a value"
                 )
             return GcsCloudStore(gcs_url)
         else:
@@ -1773,7 +1847,8 @@ def create_default_flow_state():
     @builder
     @decorators.immediate
     def core__persistent_cache(
-        core__persistent_cache__local_store, core__persistent_cache__cloud_store,
+        core__persistent_cache__local_store,
+        core__persistent_cache__cloud_store,
     ):
         return PersistentCache(
             local_store=core__persistent_cache__local_store,
@@ -1786,11 +1861,84 @@ def create_default_flow_state():
 
     @builder
     @decorators.immediate
-    def core__executor(
-        core__parallel_execution__enabled, core__parallel_execution__worker_count,
+    def core__process_executor(
+        core__parallel_execution__enabled,
+        core__parallel_execution__worker_count,
     ):
         if not core__parallel_execution__enabled:
             return None
-        return Executor(core__parallel_execution__worker_count)
+        return ProcessExecutor(core__parallel_execution__worker_count)
+
+    builder.assign("core__aip_execution__enabled", False, persist=False)
+    builder.assign("core__aip_execution__gcp_project_name", None, persist=False)
+    builder.assign(
+        "core__aip_execution__docker_image_name", "bionic:latest", persist=False
+    )
+
+    @builder
+    @decorators.immediate
+    def core__aip_execution__docker_image_uri(
+        core__aip_execution__gcp_project_name,
+        core__aip_execution__docker_image_name,
+    ):
+        image_name = core__aip_execution__docker_image_name
+        project = core__aip_execution__gcp_project_name
+
+        if image_name is None or project is None:
+            return None
+
+        return f"gcr.io/{project}/{image_name}"
+
+    @builder
+    @decorators.immediate
+    def core__aip_execution__config(
+        core__aip_execution__enabled,
+        core__aip_execution__gcp_project_name,
+        core__aip_execution__docker_image_uri,
+        core__flow_name,
+    ):
+        if not core__aip_execution__enabled:
+            return None
+        if core__aip_execution__gcp_project_name is None:
+            error_message = """
+                core__aip_execution__gcp_project_name is None, but needs a
+                value. AIP uses project to verify IAM permissions.
+            """
+            raise AssertionError(oneline(error_message))
+        if core__aip_execution__docker_image_uri is None:
+            error_message = """
+                core__aip_execution__docker_image_uri is None, but
+                needs a value. AIP uses the docker image from the
+                Container Registry to run jobs and workers.
+            """
+            raise AssertionError(oneline(error_message))
+        return AipConfig(
+            uuid=f"{core__flow_name}_{datetime.now().strftime('%Y%m%d_%H%M%S')}",
+            project_name=core__aip_execution__gcp_project_name,
+            image_uri=core__aip_execution__docker_image_uri,
+        )
+
+    @builder
+    @decorators.immediate
+    def core__aip_executor(
+        core__parallel_execution__enabled,
+        core__aip_execution__enabled,
+        core__aip_execution__config,
+    ):
+        if not core__aip_execution__enabled:
+            return None
+        # TODO: Remove this restriction once we unify the wait logic
+        # for aip and process futures in flow execution.
+        if core__parallel_execution__enabled:
+            error_message = """
+                Both core__aip_execution__enabled and
+                core__aip_execution__enabled, but only one can be
+                enabled.
+            """
+            raise AssertionError(oneline(error_message))
+        # TODO: Add checks that all the AIP libraries are installed. Otherwise,
+        # users have to wait till job submission to get the error back that the
+        # required libraries are not installed.
+        return AipExecutor(core__aip_execution__config)
 
     return builder._state.mark_all_entities_default()
