@@ -24,7 +24,7 @@ from pyarrow import parquet, Table
 import pandas as pd
 
 from .decoration import decorator_updating_accumulator
-from .exception import UnsupportedSerializedValueError
+from .exception import EntityValueError, UnsupportedSerializedValueError
 from .deps.optdep import import_optional_dependency
 from .utils.files import recursively_copy_path
 from .utils.misc import (
@@ -50,11 +50,39 @@ def check_is_like_protocol(obj):
 class BaseProtocol:
     def validate(self, value):
         """
-        Checks if a value is valid for this protocol. Throws an exeception if the value
+        Checks if a value is valid for this protocol. Throws an exception if the value
         is invalid; otherwise does nothing.
         """
         pass
 
+    def validate_for_entity(self, entity_name, value):
+        """
+        Like ``validate``, but any raised exception will be wrapped by an
+        ``EntityValueError`` with a clearer error message specific to an entity.
+        """
+        try:
+            self.validate(value)
+        except Exception as e:
+            self._raise_validation_exception(f"entity {entity_name!r}", e)
+
+    def validate_for_dnode(self, dnode, value):
+        """
+        Like ``validate``, but any raised exception will be wrapped by an
+        ``EntityValueError`` with a clearer error message specific to a descriptor.
+        """
+        try:
+            self.validate(value)
+        except Exception as e:
+            self._raise_validation_exception(repr(dnode.to_descriptor()), e)
+
+    def _raise_validation_exception(self, descriptor_text, exception):
+        message = f"""
+        Value received for {descriptor_text} is not valid for
+        {self.__class__.__name__} due to {exception.__class__}:
+        {exception}
+        """
+        raise EntityValueError(oneline(message)) from exception
+
     def value_is_valid(self, value):
         """Indicates whether the passed value is valid for this protocol."""
         try:
@@ -86,9 +114,12 @@ class BaseProtocol:
         """
         return self.get_fixed_file_extension()
 
-    def can_read_file_extension(self, extension):
-        """Indicates whether a file extension can be read by this protocol."""
-        return extension == self.get_fixed_file_extension()
+    def supports_filename(self, filename):
+        """
+        Indicates whether a filename can be deserialized by the protocol (typically by
+        examining the file extension).
+        """
+        return filename.endswith("." + self.get_fixed_file_extension())
 
     def supports_value(self, value):
         """Indicates whether a value can be serialized by this protocol."""
@@ -102,16 +133,6 @@ class BaseProtocol:
         """Deserializes an object from the pathlib path ``path``, and returns it."""
         raise NotImplementedError()
 
-    def read_with_extension(self, path, extension):
-        """
-        Deserializes an object from the pathlib path ``path``, and returns it.
-
-        May use the file extension ``extension`` to determine what deserialization
-        method to use. This extension must be the same extension generated by the
-        protocol that originally serialized the value.
-        """
-        return self.read(path)
-
     SIMPLE_TYPES = {
         bool,
         str,
@@ -608,14 +629,14 @@ class CombinedProtocol(BaseProtocol):
                 return protocol
         return None
 
-    def _protocol_for_extension(self, extension):
+    def _protocol_for_filename(self, filename):
         for protocol in self._subprotocols:
-            if protocol.can_read_file_extension(extension):
+            if protocol.supports_filename(filename):
                 return protocol
         return None
 
-    def can_read_file_extension(self, extension):
-        return self._protocol_for_extension(extension) is not None
+    def supports_filename(self, filename):
+        return self._protocol_for_filename(filename) is not None
 
     def file_extension_for_value(self, value):
         return self._protocol_for_value(value).file_extension_for_value(value)
@@ -629,20 +650,20 @@ class CombinedProtocol(BaseProtocol):
         else:
             self._subprotocols[-1].validate(value)
 
-    def read_with_extension(self, path, extension):
-        protocol = self._protocol_for_extension(extension)
+    def read(self, path):
+        protocol = self._protocol_for_filename(path.name)
         if protocol is None:
-            raise ValueError(
-                "This protocol doesn't know how to read a file with "
-                f"extension {extension!r}"
-            )
-        return protocol.read_with_extension(path, extension)
+            message = f"""
+            This protocol couldn't recognize any known file extension for file {path}
+            """
+            raise ValueError(oneline(message))
+        return protocol.read(path)
 
     def write(self, value, path):
         self._protocol_for_value(value).write(value, path)
 
     def __repr__(self):
-        return f"CombinedProtocol{tuple(self._subprotocols)!r}"
+        return "CombinedProtocol(...)"
 
 
 class TypeProtocol(PicklableProtocol):
@@ -763,3 +784,10 @@ class TupleProtocol(BaseProtocol):
             instead, got {len(items)} values: {items!r}
             """
             raise AssertionError(oneline(message))
+
+
+class NonSerializableObjectProtocol(BaseProtocol):
+    """
+    Describes values that can't be serialized or deserialized. Used mainly for
+    intermediate values in Bionic's infrastructure.
+    """
