@@ -1,18 +1,31 @@
+import re
+
 import pytest
 
 import logging
 import threading
+import bionic as bn
 
 
 class LogChecker:
     def __init__(self, caplog):
         self._caplog = caplog
 
-    def expect(self, *expected_messages):
-        actual_messages = [record.getMessage() for record in self._caplog.records]
+    def expect_all(self, *expected_messages):
+        actual_messages = self._pop_messages()
         assert set(actual_messages) == set(expected_messages)
         self._caplog.clear()
 
+    def expect_regex(self, *expected_patterns):
+        actual_messages = self._pop_messages()
+        for pattern in expected_patterns:
+            assert any(re.fullmatch(pattern, message) for message in actual_messages)
+
+    def _pop_messages(self):
+        messages = [record.getMessage() for record in self._caplog.records]
+        self._caplog.clear()
+        return messages
+
 
 @pytest.fixture(scope="function")
 def log_checker(caplog):
@@ -44,7 +57,7 @@ def test_logging_details(builder, log_checker, parallel_execution_enabled):
 
     flow = builder.build()
     assert flow.get("x_plus_one") == 2
-    log_checker.expect(
+    log_checker.expect_all(
         "Accessed   x(x=1) from definition",
         "Computing  x_plus_one(x=1) ...",
         "Computed   x_plus_one(x=1)",
@@ -56,13 +69,13 @@ def test_logging_details(builder, log_checker, parallel_execution_enabled):
         # This is different from serial execution because we don't pass
         # in-memory cache to the subprocesses. The subprocess loads the
         # entities from disk cache instead.
-        log_checker.expect(
+        log_checker.expect_all(
             "Loaded     x_plus_one(x=1) from disk cache",
             "Computing  x_plus_two(x=1) ...",
             "Computed   x_plus_two(x=1)",
         )
     else:
-        log_checker.expect(
+        log_checker.expect_all(
             "Accessed   x_plus_one(x=1) from in-memory cache",
             "Computing  x_plus_two(x=1) ...",
             "Computed   x_plus_two(x=1)",
@@ -77,15 +90,15 @@ def test_logging_details(builder, log_checker, parallel_execution_enabled):
     # To clarify: we do access it for looking at the cache, but it's
     # taken from the case key where it is loaded by default and is not
     # counted as definition access in the flow.
-    log_checker.expect("Loaded     x_plus_one(x=1) from disk cache")
+    log_checker.expect_all("Loaded     x_plus_one(x=1) from disk cache")
 
     flow = builder.build()
     assert flow.get("x_plus_two") == 3
-    log_checker.expect("Loaded     x_plus_two(x=1) from disk cache")
+    log_checker.expect_all("Loaded     x_plus_two(x=1) from disk cache")
 
     flow = flow.setting("x_plus_one", 3)
     assert flow.get("x_plus_two") == 4
-    log_checker.expect(
+    log_checker.expect_all(
         "Accessed   x_plus_one(x_plus_one=3) from definition",
         "Computing  x_plus_two(x_plus_one=3) ...",
         "Computed   x_plus_two(x_plus_one=3)",
@@ -112,9 +125,52 @@ def test_log_unpickleable_value(builder, log_checker):
 
     assert builder.build().get("log_unpickleable_value") == 5
 
-    log_checker.expect(
+    log_checker.expect_all(
         "Computing  log_unpickleable_value() ...",
         "Cannot pickle me",
         "Logging unpickleable class: Cannot pickle me",
         "Computed   log_unpickleable_value()",
     )
+
+
+@pytest.mark.no_parallel
+@pytest.mark.needs_aip
+def test_log_aip(aip_builder, log_checker):
+    builder = aip_builder
+
+    builder.assign("x", 1)
+
+    @builder
+    @bn.aip_task_config("n1-standard-4")
+    def x_plus_one(x):
+        return x + 1
+
+    @builder
+    @bn.aip_task_config("n1-standard-8")
+    def x_plus_two(x_plus_one):
+        return x_plus_one + 1
+
+    flow = builder.build()
+
+    assert flow.get("x_plus_one") == 2
+
+    log_checker.expect_regex(
+        r"Accessed   x\(x=1\) from definition",
+        r"Uploading x\(x=1\) to GCS ...",
+        r"Staging task .* at gs://.*",
+        r"Submitting .*x_plus_one.*",
+        r"Started task on AI Platform: https://console.cloud.google.com/ai-platform/jobs/.*",
+        r"Computed   x_plus_one\(x=1\) using AIP",
+    )
+
+    assert flow.get("x_plus_two") == 3
+
+    # Unfortunately, the string "Loaded     x_plus_one(x=1) from disk cache" is
+    # printed in AIP logs but not locally.
+
+    log_checker.expect_regex(
+        r"Staging task .* at gs://.*",
+        r"Submitting .*x_plus_two.*",
+        r"Started task on AI Platform: https://console.cloud.google.com/ai-platform/jobs/.*",
+        r"Computed   x_plus_two\(x=1\) using AIP",
+    )
