@@ -15,11 +15,10 @@ import tempfile
 import dask.dataframe as dd
 
 from ..helpers import (
-    count_calls,
     df_from_csv_str,
     equal_frame_and_index_content,
-    gsutil_wipe_path,
     local_wipe_path,
+    gcs_fs_wipe_path,
 )
 from bionic.exception import CodeVersioningError
 
@@ -47,7 +46,7 @@ def test_gcs_caching(preset_gcs_builder, make_counter):
     builder = preset_gcs_builder
 
     @builder
-    @count_calls(call_counter)
+    @call_counter
     def xy(x, y):
         return x * y
 
@@ -65,7 +64,7 @@ def test_gcs_caching(preset_gcs_builder, make_counter):
     assert flow.setting("x", 4).get("xy") == 12
     assert call_counter.times_called() == 0
 
-    gsutil_wipe_path(gcs_cache_url)
+    gcs_fs_wipe_path(gcs_cache_url)
     flow = builder.build()
 
     assert flow.get("xy") == 6
@@ -79,7 +78,7 @@ def test_gcs_caching(preset_gcs_builder, make_counter):
     assert flow.setting("x", 4).get("xy") == 12
     assert call_counter.times_called() == 0
 
-    gsutil_wipe_path(gcs_cache_url)
+    gcs_fs_wipe_path(gcs_cache_url)
     local_wipe_path(local_cache_path_str)
     flow = builder.build()
 
@@ -229,7 +228,7 @@ def test_multifile_serialization(preset_gcs_builder, make_counter):
 
     @builder
     @bn.protocol.dask
-    @count_calls(call_counter)
+    @call_counter
     def df():
         return dask_df
 
@@ -254,7 +253,7 @@ def test_file_path_copying(preset_gcs_builder, make_counter):
     file_contents = "DATA"
 
     @builder
-    @count_calls(call_counter)
+    @call_counter
     @bn.protocol.path(operation="move")
     def data_path():
         call_counter.mark()
