@@ -11,11 +11,12 @@ import pandas.testing as pdt
 from PIL import Image
 import dask.dataframe as dd
 
-from ..helpers import count_calls, df_from_csv_str, equal_frame_and_index_content
+from ..helpers import df_from_csv_str, equal_frame_and_index_content
 
 import bionic as bn
 from bionic.exception import (
     EntitySerializationError,
+    EntityValueError,
     UnsupportedSerializedValueError,
 )
 from bionic.protocols import CombinedProtocol, PicklableProtocol
@@ -57,7 +58,7 @@ def test_picklable_value(builder, make_counter, protocol, value):
 
     @builder
     @protocol
-    @count_calls(counter)
+    @counter
     def picklable_value():
         return value
 
@@ -73,7 +74,7 @@ def test_jsonable_value(builder, make_counter, protocol, value):
 
     @builder
     @protocol
-    @count_calls(counter)
+    @counter
     def jsonable_value():
         return value
 
@@ -90,7 +91,7 @@ def test_non_jsonable_value_fails(builder):
         circular_ref_array.append(circular_ref_array)
         return circular_ref_array
 
-    with pytest.raises(RecursionError):
+    with pytest.raises(EntityValueError):
         builder.build().get("non_jsonable_value")
 
 
@@ -99,7 +100,7 @@ def test_picklable_with_parens(builder, make_counter):
 
     @builder
     @bn.protocol.picklable()
-    @count_calls(counter)
+    @counter
     def picklable_value():
         return 1
 
@@ -114,7 +115,7 @@ def test_picklable_value_is_also_dillable(builder, make_counter, value):
 
     @builder
     @bn.protocol.dillable
-    @count_calls(counter)
+    @counter
     def dillable_value():
         return value
 
@@ -138,7 +139,7 @@ def test_dillable(builder, make_counter):
 
     @builder
     @bn.protocol.dillable
-    @count_calls(counter)
+    @counter
     def add_two():
         return make_adder(2)
 
@@ -161,7 +162,7 @@ def test_simple_dataframe(builder, make_counter):
 
     @builder
     @bn.protocol.frame
-    @count_calls(counter)
+    @counter
     def df():
         return df_value
 
@@ -290,7 +291,7 @@ def test_simple_dask_dataframe(builder, make_counter, protocol):
 
     @builder
     @protocol
-    @count_calls(counter)
+    @counter
     def df():
         return dask_df
 
@@ -318,7 +319,7 @@ def test_multiple_partitions_dask_dataframe(builder, make_counter):
 
     @builder
     @bn.protocol.dask
-    @count_calls(counter)
+    @counter
     def df():
         return dask_df
 
@@ -361,7 +362,7 @@ def test_dask_dataframe_index_cols(builder):
         Delhi,India,Asia,26
         Shanghai,China,Asia,24
         Sao Paulo,Brazil,South America,21
-        Mumbai,India,21
+        Mumbai,India,Asia,21
         Mexico City,Mexico,North America,21
         Beijing,China,Asia,20
         Osaka,Japan,Asia,20
@@ -381,8 +382,19 @@ def test_dask_dataframe_index_cols(builder):
     def counts_df(raw_df):
         return raw_df.groupby(["continent", "country"]).size().to_frame("count")
 
-    with pytest.raises(UnsupportedSerializedValueError):
-        builder.build().get("counts_df")
+    try:
+        # Versions of Dask before 2.22.0 will emit a warning here, which causes us to
+        # throw an exception. Later versions will return a multi-index frame... but the
+        # sub-indices will sometimes be in the wrong order! I don't know of a good way
+        # to catch this without calling `df.compute()`, so for now I'll just manually
+        # re-order the indices.
+        # More details in this Dask issue: https://github.com/dask/dask/issues/6484
+        # TODO Is there some other way we can catch this possible re-ordering?
+        df = builder.build().get("counts_df").compute()
+        fixed_df = df.reset_index().set_index(["continent", "country"], drop=True)
+        assert fixed_df.loc["Asia"].loc["Japan"]["count"] == 2
+    except UnsupportedSerializedValueError:
+        pass
 
 
 @pytest.mark.parametrize("protocol", [bn.protocol.image, passthrough])
@@ -488,11 +500,11 @@ def test_type_protocol(builder):
     builder.set("float_val", 1.0)
     builder.set("str_val", "one")
 
-    with pytest.raises(AssertionError):
+    with pytest.raises(EntityValueError):
         builder.set("int_val", "one")
-    with pytest.raises(AssertionError):
+    with pytest.raises(EntityValueError):
         builder.set("float_val", 1)
-    with pytest.raises(AssertionError):
+    with pytest.raises(EntityValueError):
         builder.set("str_val", 1.0)
 
     flow = builder.build()
@@ -511,7 +523,7 @@ def test_enum_protocol(builder):
     builder.set("color", "blue")
     assert builder.build().get("color") == "blue"
 
-    with pytest.raises(AssertionError):
+    with pytest.raises(EntityValueError):
         builder.set("color", "green")
 
 
@@ -529,7 +541,7 @@ def test_path_protocol(builder, make_counter, tmp_path, operation):
 
     @builder
     @bn.protocol.path(operation=operation)
-    @count_calls(phrase_file_path_counter)
+    @phrase_file_path_counter
     def phrase_file_path():
         "A path to a file containing a phrase."
         output_phrase_file_path.write_text(phrase_str)
@@ -539,7 +551,7 @@ def test_path_protocol(builder, make_counter, tmp_path, operation):
 
     @builder
     @bn.protocol.path(operation=operation)
-    @count_calls(colors_dir_path_counter)
+    @colors_dir_path_counter
     def colors_dir_path():
         "A path to a directory containing some files named after colors."
         output_colors_dir_path.mkdir()
@@ -621,18 +633,18 @@ def test_combined_protocol(builder):
     flow = builder.build().setting("value", 1)
 
     assert flow.get("must_be_one") == "one"
-    with pytest.raises(AssertionError):
+    with pytest.raises(EntityValueError):
         flow.get("must_be_two")
     assert flow.get("must_be_one_or_two") == "one"
 
     flow = flow.setting("value", 2)
-    with pytest.raises(AssertionError):
+    with pytest.raises(EntityValueError):
         flow.get("must_be_one")
     assert flow.get("must_be_two") == "two"
     assert flow.get("must_be_one_or_two") == "two"
 
     flow = flow.setting("value", 3)
-    with pytest.raises(AssertionError):
+    with pytest.raises(EntityValueError):
         flow.get("must_be_one_or_two")
 
 
